// Project Vision Quake III Viewer
// note: PC version of these shaders has been
// drastically simplified to avoid creating too
// complex shaders.

struct VTN_VERTEX_IN
{
	float4 position : POSITION;
	float4 normal : NORMAL;
	float4 color : COLOR;
	float2 tex0 : TEXCOORD0;
	float3 tex1 : TEXCOORD1;
};

struct VTN_VERTEX_OUT
{
	float4 position : POSITION;
	float4 color : COLOR;
	float2 tex0 : TEXCOORD0;
	float3 tex1 : TEXCOORD1;	
};

struct DEF_VTN_VERTEX_IN
{
	float4 position : POSITION;
	float4 color : COLOR;
	float2 tex0 : TEXCOORD0;
};

float4x4 mvp : register(c0);

DEF_VTN_VERTEX_IN def_main_vs ( DEF_VTN_VERTEX_IN input )
{
	return input;
}

VTN_VERTEX_OUT q3_main_vs ( VTN_VERTEX_IN input )
{
	VTN_VERTEX_OUT output;

	output.position 	= mul(mvp, input.position);
	output.color 	= input.color;
	output.tex0 	= input.tex0;
	output.tex1 	= input.tex1;

	return output;
}

struct VTN_PIXEL_OUT
{
	float4 color : COLOR0;
};

sampler2D baseMap : register ( s0 );
sampler2D lightMap : register( s1 );
sampler2D motion2 : register ( s2 );
sampler2D motion3 : register ( s3 );
sampler2D motion4 : register ( s4 );
sampler2D motion5 : register ( s5 );
sampler2D motion6 : register ( s6 );
sampler2D motion7 : register ( s7 );

VTN_PIXEL_OUT q3_main_ps ( VTN_VERTEX_OUT input )
{
	VTN_PIXEL_OUT output;

	float3 baseColor = tex2D(baseMap, input.tex0).xyz;
	float3 lightColor = tex2D(lightMap, input.tex1).xyz;
	float3 mx = lightColor * baseColor;
	output.color.xyz = mx + mx + mx + mx;
	output.color.w = 1;

	return output;
}

// fixme: add sky_main_vs and use texCUBE to lookup
// skybox. 
VTN_PIXEL_OUT sky_main_ps( VTN_VERTEX_OUT input )
{
	VTN_PIXEL_OUT output;

	float3 baseColor = tex2D(baseMap, input.tex0).xyz;
	output.color.xyz = baseColor * input.color;
	output.color.w = 1;

	return output;
}

VTN_PIXEL_OUT hdr_pre_ps ( VTN_VERTEX_OUT input )
{
	VTN_PIXEL_OUT output;

	float3 lightColor = tex2D(lightMap, input.tex1).xyz;
	lightColor = max(0, lightColor-0.5)*2.0;
	output.color.xyz = lightColor;
	output.color.w = 1;

	return output;
}

VTN_PIXEL_OUT hdr_post_ps ( VTN_VERTEX_OUT input )
{
	VTN_PIXEL_OUT output;

	float3 baseColor = tex2D(baseMap, input.tex0).xyz;
	
      float2 hdrLookup = input.tex0;
	float3 hdrColor = tex2D(lightMap, hdrLookup).xyz;

	for ( float i = 1; i < 2; i++ )
	{
		hdrLookup = float2( input.tex0.x + i * 0.01, input.tex0.y + i * 0.01);
		hdrColor += tex2D(lightMap, hdrLookup);

		hdrLookup = float2( input.tex0.x - i * 0.01, input.tex0.y - i * 0.01);
		hdrColor += tex2D(lightMap, hdrLookup);

		hdrLookup = float2( input.tex0.x + i * 0.01, input.tex0.y - i * 0.01);
		hdrColor += tex2D(lightMap, hdrLookup);

		hdrLookup = float2( input.tex0.x - i * 0.01, input.tex0.y + i * 0.01);
		hdrColor += tex2D(lightMap, hdrLookup);

		hdrLookup = float2( input.tex0.x, input.tex0.y + i * 0.01);
		hdrColor += tex2D(lightMap, hdrLookup);

		hdrLookup = float2( input.tex0.x, input.tex0.y - i * 0.01);
		hdrColor += tex2D(lightMap, hdrLookup);

		hdrLookup = float2( input.tex0.x + i * 0.01, input.tex0.y );
		hdrColor += tex2D(lightMap, hdrLookup);

		hdrLookup = float2( input.tex0.x - i * 0.01, input.tex0.y );
		hdrColor += tex2D(lightMap, hdrLookup);
	}

	hdrColor /= 9.0;

	output.color.xyz = baseColor.xyz + hdrColor.xyz;
	output.color.w = 1;

	return output;
}

// 
// Depth of Field Effect
//
// desc: this really should be done two pass using hardware scaling
//       to avoid the ugly blockiness of the blur, but since we want
//       to keep each effect down to 1 pass max, this will suffice
//

VTN_PIXEL_OUT depthoffield_ps ( VTN_VERTEX_OUT input )
{
	VTN_PIXEL_OUT output;

	float3 baseColor = tex2D(baseMap, input.tex0).xyz;
	float4 depthColor = tex2D(lightMap, input.tex0);
	float4 tempDepth;
	float3 finalColor = baseColor;
	float2 dofLookup = float2(1,1);
	float  totalCount = 1;

	// figure out depth at this pixel then blur based on it
	float depthValue = depthColor.r;	

	if ( depthValue > 0.9 )
	{
		float i = (depthValue - 0.9) / 0.35;

		dofLookup = float2( input.tex0.x + i * 0.01, input.tex0.y + i * 0.01);
		//tempDepth = tex2D(lightMap, dofLookup );
		//if ( tempDepth.r < (depthValue + 0.03125) && tempDepth.r > (depthValue - 0.03125) )
		{
			finalColor += tex2D(baseMap, dofLookup );// * sqrt((i) * (i) * 2);
		//	totalCount = totalCount + 1;
		}

		// sample 2
		dofLookup = float2( input.tex0.x - i * 0.01, input.tex0.y - i * 0.01);
		//tempDepth = tex2D(lightMap, dofLookup );
		//if ( tempDepth.r < (depthValue + 0.03125) && tempDepth.r > (depthValue - 0.03125) )
		{
			finalColor += tex2D(baseMap, dofLookup );// * sqrt((i) * (i) * 2);
		//	totalCount = totalCount + 1;
		}
	
		// sample 3
		dofLookup = float2( input.tex0.x + i * 0.01, input.tex0.y - i * 0.01);
		//tempDepth = tex2D(lightMap, dofLookup );
		//if ( tempDepth.r < (depthValue + 0.03125) && tempDepth.r > (depthValue - 0.03125) )
		{
			finalColor += tex2D(baseMap, dofLookup );// * sqrt((i) * (i) * 2) ;
		//	totalCount = totalCount + 1;
		}
	
		// sample 4
		dofLookup = float2( input.tex0.x - i * 0.01, input.tex0.y + i * 0.01);
		//tempDepth = tex2D(lightMap, dofLookup );
		//if ( tempDepth.r < (depthValue + 0.03125) && tempDepth.r > (depthValue - 0.03125) )
		{
			finalColor += tex2D(baseMap, dofLookup );// * sqrt((i) * (i) * 2);
		//	totalCount = totalCount + 1;
		}

		// sample 5
		dofLookup = float2( input.tex0.x, input.tex0.y + i * 0.01);
		//tempDepth = tex2D(lightMap, dofLookup );
		//if ( tempDepth.r < (depthValue + 0.03125) && tempDepth.r > (depthValue - 0.03125) )
		{
			finalColor += tex2D(baseMap, dofLookup );// * sqrt((i) * (i));
		//	totalCount = totalCount + 1;
		}

		// sample 6
		dofLookup = float2( input.tex0.x, input.tex0.y - i * 0.01);
		//tempDepth = tex2D(lightMap, dofLookup );
		//if ( tempDepth.r < (depthValue + 0.03125) && tempDepth.r > (depthValue - 0.03125) )
		{
			finalColor += tex2D(baseMap, dofLookup );// * sqrt((i) * (i));
		//	totalCount = totalCount + 1;
		}
	
		// sample 7
		dofLookup = float2( input.tex0.x + i * 0.01, input.tex0.y );
		//tempDepth = tex2D(lightMap, dofLookup );
		//if ( tempDepth.r < (depthValue + 0.03125) && tempDepth.r > (depthValue - 0.03125) )
		{
			finalColor += tex2D(baseMap, dofLookup );// * sqrt((i) * (i));
		//	totalCount = totalCount + 1;
		}

		// sample 8
		dofLookup = float2( input.tex0.x - i * 0.01, input.tex0.y );
		//tempDepth = tex2D(lightMap, dofLookup );
		//if ( tempDepth.r < (depthValue + 0.03125) && tempDepth.r > (depthValue - 0.03125) )
		{
			finalColor += tex2D(baseMap, dofLookup );// * sqrt((i) * (i));
		//	totalCount = totalCount + 1;
		}

		finalColor /= 9; // totalCount;
	}	
	
	output.color.xyz = finalColor.xyz;
	output.color.w = 1;

	return output;
}

VTN_PIXEL_OUT blur_post_ps ( VTN_VERTEX_OUT input )
{
	VTN_PIXEL_OUT output;

	float3 baseColor = tex2D(baseMap, input.tex0).xyz;

	// now we blend in motion2,3,4,5,6,7
	baseColor += tex2D(motion2, input.tex0).xyz;
	baseColor += tex2D(motion3, input.tex0).xyz;
	baseColor += tex2D(motion4, input.tex0).xyz;
	baseColor += tex2D(motion5, input.tex0).xyz;
	baseColor += tex2D(motion6, input.tex0).xyz;
	baseColor += tex2D(motion7, input.tex0).xyz;

	baseColor /= 8.0;

	output.color.xyz = baseColor.xyz;
	output.color.w = 1;

	return output;
}

VTN_PIXEL_OUT blur_hdr_post_ps ( VTN_VERTEX_OUT input )
{
	VTN_PIXEL_OUT output;

	float3 baseColor = tex2D(baseMap, input.tex0).xyz;
	
    float2 hdrLookup = input.tex0;
	float3 hdrColor = tex2D(lightMap, hdrLookup).xyz;

	for ( float i = 1; i < 2; i++ )
	{
		hdrLookup = float2( input.tex0.x, input.tex0.y + i * 0.01);
		hdrColor += tex2D(lightMap, hdrLookup);

		hdrLookup = float2( input.tex0.x, input.tex0.y - i * 0.01);
		hdrColor += tex2D(lightMap, hdrLookup);

		hdrLookup = float2( input.tex0.x + i * 0.01, input.tex0.y );
		hdrColor += tex2D(lightMap, hdrLookup);

		hdrLookup = float2( input.tex0.x - i * 0.01, input.tex0.y );
		hdrColor += tex2D(lightMap, hdrLookup);
	}

	hdrColor /= 5.0;

	// now we blend in motion2,3,4,5,6,7
	hdrColor += tex2D(motion2, input.tex0).xyz;
	hdrColor += tex2D(motion3, input.tex0).xyz;
	hdrColor += tex2D(motion4, input.tex0).xyz;
	hdrColor += tex2D(motion5, input.tex0).xyz;
	hdrColor += tex2D(motion6, input.tex0).xyz;
	hdrColor += tex2D(motion7, input.tex0).xyz;
	hdrColor /= 8.0;

	output.color.xyz = baseColor.xyz + hdrColor.xyz;
	output.color.w = 1;

	return output;
}


VTN_PIXEL_OUT def_main_ps ( DEF_VTN_VERTEX_IN input )
{
	VTN_PIXEL_OUT output;

	float4 baseColor = tex2D(baseMap, input.tex0);
	output.color = baseColor * input.color;

	return output;
}